@@ -1,109 +1,85 @@
 import os
-import json
 import warnings
 
-from jsonschema import validate, ValidationError
-from .. import constants
-from .asset import Asset
+from ..databrary.recordFactory import RecordFactory
+from ..databrary.asset import Asset
+from ..databrary.volume import Volume
 
 from ..utils import utils
-from .container import Container
-from .record import Record
-from .types.category import Category
+from ..databrary.container import Container
+from ..databrary.types.category import Category
+from ..api.pybrary import Pybrary
 
 
 class Curation:
-    def __init__(self, volume_name, containers_file, assets_file=None, records_file=None):
+    @staticmethod
+    def from_databrary(username, password, volume_id, superuser=False, check_errors=True):
+        pb = Pybrary.get_instance(username, password, superuser)
+        volume_data = pb.get_volume_info(volume_id)
+
+        if check_errors:
+            errors = Curation.check_errors(volume_data, volume_id)
+            if errors:
+                raise Exception('Errors in volume {}, Check {}'.format(volume_id, errors))
+            else:
+                print('No error found in volume {}'.format(volume_id))
+
+        return Volume.from_databrary(volume_data)
+
+    @staticmethod
+    def check_errors(volume_data, volume_id):
+        errors = dict()
+        containers = volume_data.get('containers')
+        for container in containers:
+            container_id = container.get('id')
+            for asset in container.get('assets'):
+                asset_id = asset.get('id')
+                if asset.get('name') is None:
+                    error = 'Missing name for Asset {}: Please update the name here: https://nyu.databrary.org/volume/{}/slot/{}?asset={}'.format(asset_id, volume_id, container_id, asset_id)
+                    if container_id in errors:
+                        errors.get(container_id).append(error)
+                    else:
+                        errors[container_id] = [error]
+        return errors
+
+    @staticmethod
+    def from_files(volume_name, containers_file, assets_file=None, records_file=None):
         if volume_name is None:
             raise AttributeError('You must provide a valid volume name.')
-        self._volume_name = volume_name
+        else:
+            name = volume_name
+
         if containers_file is None:
             raise AttributeError('Containers file is required')
+        else:
+            containers = Curation.read_containers_csv(containers_file)
 
         if assets_file is None:
             warnings.warn('Assets found in the containers file will be ignored. PLease provide an assets file')
         else:
-            self._assets = Curation.read_assets_csv(assets_file)
+            assets = Curation.read_assets_csv(assets_file)
 
         if records_file is None:
             warnings.warn('Records found in the containers file will be ignored. PLease provide a records file')
         else:
-            self._records = Curation.read_records_csv(records_file)
+            records = Curation.read_records_csv(records_file)
 
-        self._containers = Curation.read_containers_csv(containers_file)
+        return Volume(name, containers, assets, records)
 
-    def add_assets(self, file_path):
-        self._assets = Curation.read_assets_csv(file_path)
-
-    def add_records(self, file_path):
-        self._records = Curation.read_records_csv(file_path)
-
-    def _ignore_assets(self):
-        return self._assets is None
-
-    def _ignore_records(self):
-        return self._records is None
-
-    def _find_asset(self, asset_id):
-        if self._ignore_assets():
-            return None
-
-        for asset in self._assets:
-            if asset.get_id() == asset_id:
-                return asset
-
-        return None
-
-    def _find_record(self, record_id):
-        if self._ignore_assets():
-            return None
-
-        for record in self._records:
-            if record.get_id() == record_id:
-                return record
-
-        return None
-
-    def generate_ingest_file(self, output):
+    @staticmethod
+    def get_ingest_file(volume, output):
         if output is None:
-            raise IOError('Please provide an output')
+            raise IOError('Please provide an output path')
         if utils.get_file_extension(output) != 'json':
             output = utils.replace_file_extension(output)
             warnings.warn('Ingest files must be in JSON format, ingest file will be saved in {}'
                           .format(output))
-        result = {
-            "name": self._volume_name,
-            "containers": []
-        }
 
-        for container in self._containers:
-            container_dict = container.to_dict()
-            asset_ids = container.get_assets()
-            container_assets = []
-            for asset_id in asset_ids:
-                asset = self._find_asset(asset_id)
-                container_assets.append(asset.to_dict())
+        utils.dump_into_json(volume.to_dict(), output)
 
-            container_dict['assets'] = container_assets
-
-            record_ids = container.get_records()
-            container_records = []
-            for record_id in record_ids:
-                record = self._find_record(record_id)
-                container_records.append(record.to_dict())
-
-            container_dict['records'] = container_records
-
-            result.get('containers').append(container_dict)
-        with open(constants.VOLUME_SCHEMA_FILE) as f:
-            schema = json.loads(f.read())
-        try:
-            validate(result, schema)
-        except ValidationError as e:
-            raise Exception('Did not pass validation against volume.json schema - Errors: {}'.format(e))
-
-        utils.dump_into_json(result, output)
-        return result
+    @staticmethod
+    def get_files_from_volume(volume, dir_path):
+        pass
 
     @staticmethod
     def generate_assets_list(folder, output=None):
@@ -157,9 +133,10 @@ class Curation:
 
             for i in range(value):
                 records.append(
-                    Record(
+                    RecordFactory(
                         category=Category.get_name(category),
                         key=idx,
+                        id=idx
                     )
                 )
                 idx = idx + 1
@@ -193,6 +170,7 @@ class Curation:
                 result.append(data.to_dict(template=template))
             except Exception as e:
                 print('Error {}'.format(e))
+
         utils.dump_into_csv(result, output)
 
     @staticmethod
@@ -207,8 +185,8 @@ class Curation:
                               .format(id, file_path, record_dict.get('name')))
                 continue
             ids[id] = None
-            record = Record.from_dict(record_dict)
-            records.append(record)
+            records.append(RecordFactory.from_dict(record_dict))
+
         return records
 
     @staticmethod
@@ -225,6 +203,7 @@ class Curation:
             ids[id] = None
             asset = Asset.from_dict(asset_dict)
             assets.append(asset)
+
         return assets
 
     @staticmethod
@@ -241,6 +220,7 @@ class Curation:
             ids[id] = None
             container = Container.from_dict(container_dict)
             containers.append(container)
+
         return containers
 
     @staticmethod
@@ -251,8 +231,5 @@ class Curation:
         :param target: Target Volume ID
         :return:
         """
-        return "COPY (select 'mkdir -p /nyu/stage/reda/' || '" + str(target) + "' || '/' || '" + str(target)
-        + "' || sa.container || ' && ' || E'cp \"/nyu/store/' || substr(cast(sha1 as varchar(80)), 3, 2) || '/' || right(cast(sha1 as varchar(80)), -4) || '" "/nyu/stage/reda/' || '"
-        + str(target) + "' || '/' || '" + str(target)
-        + "' || container || '/' || CASE WHEN a.name LIKE '%.___' IS FALSE THEN a.name || '.' || f.extension[1] ELSE a.name END || E'\"' from slot_asset sa inner join asset a on sa.asset = a.id inner join format f on a.format = f.id where a.volume = "
-        + str(source) + ") TO '/tmp/volume_" + str(target) + ".sh';"
+        return "COPY (" \
+               "select 'mkdir -p /nyu/stage/reda/' || '" + str(source) + "' || '/' || sa.container || ' && ' || E'cp \"/nyu/store/' || substr(cast(sha1 as varchar(80)), 3, 2) || '/' || right(cast(sha1 as varchar(80)), -4) || '\" \"' || '/nyu/stage/reda/' || '" + str(source) + "' || '/' || sa.container || '/' || CASE WHEN a.name LIKE '%.___' IS FALSE THEN a.name || '.' || f.extension[1] || '\"' ELSE a.name || '\"' END from slot_asset sa inner join asset a on sa.asset = a.id inner join format f on a.format = f.id where a.volume = " + str(source) + ") TO '/tmp/volume_" + str(target) + ".sh';"
